'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _classCallCheck2 = require('babel-runtime/helpers/classCallCheck');

var _classCallCheck3 = _interopRequireDefault(_classCallCheck2);

var _createClass2 = require('babel-runtime/helpers/createClass');

var _createClass3 = _interopRequireDefault(_createClass2);

var _hessian = require('hessian.js');

var _utils = require('./utils');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

// 8 * 1024 * 1024 default body max length
var DEFAULT_LEN = 8388608;

var ENUM_TYPE = {
  boolean: 'Z',
  int: 'I',
  short: 'S',
  long: 'J',
  double: 'D',
  float: 'F'
};

var Encoder = function () {
  function Encoder(_ref) {
    var dubboVersion = _ref.dubboVersion,
        _interface = _ref.interface,
        version = _ref.version,
        group = _ref.group,
        timeout = _ref.timeout;
    (0, _classCallCheck3.default)(this, Encoder);


    this._dubboVersion = dubboVersion;
    this._interface = _interface;
    this._version = version;
    this._group = group;
    this._timeout = timeout;
    this._gte280 = (0, _utils.gte)(dubboVersion, '2.8.0');

    var implicitArgs = {
      interface: _interface,
      path: _interface,
      timeout: timeout
    };

    if (version) {
      implicitArgs.version = version;
    }

    if (group) {
      implicitArgs.group = group;
    }

    this._attachments = {
      $class: 'java.util.HashMap',
      $: implicitArgs
    };
  }

  (0, _createClass3.default)(Encoder, [{
    key: 'encode',
    value: function encode(method, args) {
      var body = this._body(method, args);
      var head = this._head(body.length);
      return Buffer.concat([head, body]);
    }
  }, {
    key: '_head',
    value: function _head(len) {
      var head = [0xda, 0xbb, 0xc2, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0];

      if (len > DEFAULT_LEN) {
        throw new Error('Data length too large: ' + len + ', max payload: ' + DEFAULT_LEN);
      }

      var i = 15;
      while (len >= 256) {
        head.splice(i--, 1, len % 256);
        len >>= 8;
      }
      head.splice(i, 1, len);

      return new Buffer(head);
    }
  }, {
    key: '_body',
    value: function _body(method, args) {
      var body = new _hessian.EncoderV2();

      body.write(this._dubboVersion);
      body.write(this._interface);
      body.write(this._version);
      body.write(method);

      if (this._gte280) {
        body.write(-1);
      }

      body.write(this._argsType(args));

      args.forEach(function (arg) {
        body.write(args);
      });

      body.write(this._attachments);

      return body.byteBuffer._bytes.slice(0, body.byteBuffer._offset);
    }
  }, {
    key: '_argsType',
    value: function _argsType(args) {
      if (!args.length) {
        return '';
      }

      var ret = '';

      args.forEach(function (arg) {
        var type = arg.$class;

        ret += type.charAt(0) === '[' ? ~type.indexOf('.') ? '[L' + type.slice(1).replace(/\./gi, '/') + '' : '[' + ENUM_TYPE[type.slice(1)] : type && ~type.indexOf('.') ? 'L' + type.replace(/\./gi, '/') + '' : ENUM_TYPE[type];
      });

      return ret;
    }
  }]);
  return Encoder;
}();

exports.default = Encoder;
module.exports = exports['default'];