'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _toConsumableArray2 = require('babel-runtime/helpers/toConsumableArray');

var _toConsumableArray3 = _interopRequireDefault(_toConsumableArray2);

var _keys = require('babel-runtime/core-js/object/keys');

var _keys2 = _interopRequireDefault(_keys);

var _promise = require('babel-runtime/core-js/promise');

var _promise2 = _interopRequireDefault(_promise);

var _getPrototypeOf = require('babel-runtime/core-js/object/get-prototype-of');

var _getPrototypeOf2 = _interopRequireDefault(_getPrototypeOf);

var _classCallCheck2 = require('babel-runtime/helpers/classCallCheck');

var _classCallCheck3 = _interopRequireDefault(_classCallCheck2);

var _createClass2 = require('babel-runtime/helpers/createClass');

var _createClass3 = _interopRequireDefault(_createClass2);

var _possibleConstructorReturn2 = require('babel-runtime/helpers/possibleConstructorReturn');

var _possibleConstructorReturn3 = _interopRequireDefault(_possibleConstructorReturn2);

var _inherits2 = require('babel-runtime/helpers/inherits');

var _inherits3 = _interopRequireDefault(_inherits2);

var _utils = require('./utils');

var _encoder = require('./encoder');

var _encoder2 = _interopRequireDefault(_encoder);

var _decode = require('./decode');

var _decode2 = _interopRequireDefault(_decode);

var _events = require('events');

var _querystring = require('querystring');

var _querystring2 = _interopRequireDefault(_querystring);

var _url = require('url');

var _url2 = _interopRequireDefault(_url);

var _net = require('net');

var _net2 = _interopRequireDefault(_net);

var _debug2 = require('debug');

var _debug3 = _interopRequireDefault(_debug2);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var debug = (0, _debug3.default)('dubbo');

var CREATE_MODES = {

  // The znode will not be automatically deleted upon client's disconnect.
  PERSISTENT: 0,

  // The znode will not be automatically deleted upon client's disconnect,
  // and its name will be appended with a monotonically increasing number.
  PERSISTENT_SEQUENTIAL: 2,

  // The znode will be deleted upon the client's disconnect.
  EPHEMERAL: 1,

  // The znode will be deleted upon the client's disconnect, and its name
  // will be appended with a monotonically increasing number.
  EPHEMERAL_SEQUENTIAL: 3
};

var Service = function (_EventEmitter) {
  (0, _inherits3.default)(Service, _EventEmitter);

  function Service(name) {
    (0, _classCallCheck3.default)(this, Service);

    var _this = (0, _possibleConstructorReturn3.default)(this, (Service.__proto__ || (0, _getPrototypeOf2.default)(Service)).call(this));

    _this._name = name;
    _this._hosts = [];
    _this._methods = {};
    _this._setupError = null;
    _this._ready = false;
    return _this;
  }

  // Register service informations


  (0, _createClass3.default)(Service, [{
    key: 'register',
    value: function register(_ref) {
      var zookeeper = _ref.zookeeper,
          _ref$options = _ref.options,
          _interface = _ref$options.interface,
          version = _ref$options.version,
          group = _ref$options.group,
          _ref$options$timeout = _ref$options.timeout,
          timeout = _ref$options$timeout === undefined ? 6000 : _ref$options$timeout,
          dubboVersion = _ref.dubboVersion,
          _ref$path = _ref.path,
          consumer = _ref$path.consumer,
          provider = _ref$path.provider;


      if (this._registered) {
        return;
      }

      this._registered = true;

      this._zookeeper = zookeeper;
      this._version = version;
      this._group = group;
      this._consumer = consumer;
      this._provider = provider;
      this._err = null;
      this._encoder = new _encoder2.default({
        dubboVersion: dubboVersion,
        interface: _interface,
        version: version,
        group: group,
        timeout: timeout
      });
    }

    // Setup service

  }, {
    key: 'setup',
    value: function setup() {
      var _this2 = this;

      return _promise2.default.all([this._createNode(), this._initProvider()]).then(function () {
        _this2._ready = true;
        _this2.emit('ready');
      }).catch(function (err) {
        _this2._setupError = err;
        _this2._ready = true;
        _this2.emit('ready');
      });
    }

    // @returns `Promise`

  }, {
    key: '_createNode',
    value: function _createNode() {
      var zk = this._zookeeper;
      var consumer = this._consumer;

      return new _promise2.default(function (resolve, reject) {
        zk.exists(consumer, function (err, stat) {
          if (err) {
            return reject(err);
          }

          if (stat) {
            return resolve();
          }

          zk.create(consumer, CREATE_MODES.EPHEMERAL, function (err, node) {
            if (err) {
              return reject(err);
            }

            resolve();
          });
        });
      });
    }
  }, {
    key: '_initProvider',
    value: function _initProvider() {
      var _this3 = this;

      return new _promise2.default(function (resolve, reject) {
        _this3._zookeeper.getChildren(_this3._provider, function () {
          return _this3._initProvider();
        }, function (err, children) {
          if (err) {
            err.code = 'ERR_GET_CHILDREN';
            return reject(err);
          }

          resolve(children);
        });
      }).then(function (children) {
        return _this3._applyProviders(children);
      });
    }
  }, {
    key: '_applyProviders',
    value: function _applyProviders(children) {
      var _this4 = this;

      if (!children) {
        return (0, _utils.reject)('no children found for "' + this._provider + '"', 'NO_CHILDREN');
      }

      var available = children.map(function (child) {
        return _querystring2.default.parse(decodeURIComponent(child));
      }).filter(function (zoo) {
        return zoo.version === _this4._version && zoo.group === _this4._group;
      });

      if (!available.length) {
        return (0, _utils.reject)('no available children found for "' + this._provider + '"', 'NO_AVAILABLE_CHILDREN');
      }

      this._hosts = available.map(function (zoo) {
        return _url2.default.parse((0, _keys2.default)(zoo)[0]).host;
      });

      var methodsHash = {};
      available.forEach(function (zoo) {
        zoo.methods.split(',').forEach(function (method) {
          methodsHash[method] = true;
        });
      });

      this._methods = methodsHash;
    }
  }, {
    key: '_selectHost',
    value: function _selectHost() {
      return this._hosts[Math.random() * this._hosts.length | 0].split(':').reverse();
    }
  }, {
    key: 'invoke',
    value: function invoke(method) {
      var _this5 = this;

      for (var _len = arguments.length, args = Array(_len > 1 ? _len - 1 : 0), _key = 1; _key < _len; _key++) {
        args[_key - 1] = arguments[_key];
      }

      if (!this._registered) {
        return (0, _utils.reject)('service "this._name" must be registered before use', 'SERVICE_NOT_REGISTERED');
      }

      if (this._ready) {
        return this._invoke(method, args);
      }

      return new _promise2.default(function (resolve, reject) {
        _this5.once('ready', function () {
          _this5._invoke(method, args).then(resolve, reject);
        });
      });
    }
  }, {
    key: '_invoke',
    value: function _invoke(method, args) {
      var _this6 = this;

      if (this._setupError) {
        return _promise2.default.reject(this._setupError);
      }

      if (!this._methods[method]) {
        return (0, _utils.reject)('"' + method + '" is not an available method');
      }

      var buffer = this._encoder.encode(method, args);

      return new _promise2.default(function (resolve, reject) {
        var client = new _net2.default.Socket();
        var chunks = [];
        var heap = void 0;
        var bl = 16;

        client.connect.apply(client, (0, _toConsumableArray3.default)(_this6._selectHost()).concat([function () {
          client.write(buffer);
        }]));

        client.on('error', function (err) {
          var _this7 = this;

          this._initProvider().then(function () {
            client.connect.apply(client, (0, _toConsumableArray3.default)(_this7._selectHost()).concat([function () {
              client.write(buffer);
            }]));
          }, reject);
        });

        client.on('data', function (chunk) {
          if (!chunks.length) {
            var arr = Array.prototype.slice.call(chunk.slice(0, 16));
            var i = 0;

            while (i < 3) {
              bl += arr.pop() * Math.pow(256, i++);
            }
          }

          chunks.push(chunk);
          heap = Buffer.concat(chunks);

          if (heap.length >= bl) {
            client.destroy();
          }
        });

        client.on('close', function (err) {
          if (err) {
            return reject(err);
          }

          (0, _decode2.default)(heap).then(resolve, reject);
        });
      });
    }
  }]);
  return Service;
}(_events.EventEmitter);

exports.default = Service;
module.exports = exports['default'];